﻿using Jypeli;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Lumiukko, jota voi lyödä
/// </summary>
public class B1_Lumiukko : PhysicsGame
{
    /// <summary>
    /// Pistetään peli käyntiin ja määritellään kuuntelijat
    /// </summary>
    public override void Begin()
    {
        Camera.ZoomToLevel();
        Level.Background.Color = Color.Black;
        Level.CreateBorders();

        PhysicsObject alinPallo = new PhysicsObject(2 * 100.0, 2 * 100.0, Shape.Circle);
        alinPallo.Y = Level.Bottom + 200.0;
        Add(alinPallo);

        PhysicsObject keskipallo = new PhysicsObject(2 * 50.0, 2 * 50.0, Shape.Circle);
        keskipallo.Y = alinPallo.Y + 100 + 50;
        Add(keskipallo);

        PhysicsObject ylinPAllo = new PhysicsObject(2 * 30.0, 2 * 30.0, Shape.Circle);
        ylinPAllo.Y = keskipallo.Y + 50 + 30;
        Add(ylinPAllo);

        PhysicsObject p4 = PhysicsObject.CreateStaticObject(100, 100);
        p4.Position = new Vector(300, 300);
        Add(p4);

        PhysicsStructure ukko = new PhysicsStructure(alinPallo, keskipallo, ylinPAllo);
        ukko.Softness = 1;
        Add(ukko);

        Gravity = new Vector(0, -400);

        ylinPAllo.Hit(new Vector(80, 1));

        // Muutetaan painovoimaa 5 sekunnin välein
        // Timer t = new Timer();
        // t.Interval = 5.0;
        // t.Timeout += VaihdaPainovoimaa;
        // t.Start();

        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä näppäinohjeet");

        Keyboard.Listen(Key.Up, ButtonState.Pressed, LyoPalloa, "Lyö keskipalloa ylöspäin", keskipallo, new Vector(0, 1000));
        Keyboard.Listen(Key.Left, ButtonState.Pressed, LyoPalloa, "Lyö keskipalloa vasemmalle", keskipallo, new Vector(-1000, 0));
        Keyboard.Listen(Key.Right, ButtonState.Pressed, LyoPalloa, "Lyö keskipalloa oikealle", keskipallo, new Vector(1000, 0));

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Käännetään painovoimaa 90 astetta.
    /// </summary>
    private void VaihdaPainovoimaa()
    {
        Gravity = Vector.FromLengthAndAngle(Gravity.Magnitude, Gravity.Angle + Angle.FromDegrees(90.0));
    }

    /// <summary>
    /// Annetaan voimaimpulssi parametrina tulevalle
    /// fysiikkaoliolle.
    /// </summary>
    /// <param name="pallo">Pallo jota lyödään.</param>
    /// <param name="suunta">Mihin suuntaan lyödään.</param>
    private void LyoPalloa(PhysicsObject pallo, Vector suunta)
    {
        pallo.Hit(suunta);
    }
}
