using Jypeli;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Demo 7, tehtävät 4-6.
/// Piirretään palloja annettujen vektoreiden (tason pisteiden)
/// avulla. Sitten klikataan hiirellä johonkin kohtaan ja etsitään
/// lähin piste.
/// </summary>
public class T456_Pallot : PhysicsGame
{
    public override void Begin()
    {
        SetWindowSize(800, 600);
        Level.Size = new Vector(800, 600);
        PhysicsObject[] pallot = TeeSatunnaisetPallot(20, 30.0);

        GameObject munPallo = new GameObject(20, 20, Shape.Circle);
        Add(munPallo);
        munPallo.Position = new Vector(-100, -200);
        munPallo.Color = Color.Blue;

        // TAPA 1: 
        // =======
        // Annetaan hiiren kuuntelijan luomisen yhteydessä
        // argumenttina myös klikkauksen yhteydessä suoritettava
        // aliohjelma. Tämä aliohjelman argumenttina antaminen tapahtuu
        // delegate-avainsanalla. 

        PhysicsObject lahinPallo = LahinPallo(pallot, munPallo.Position);
        lahinPallo.Color = Color.Red;

        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, delegate ()
        {
            // Siirretään ensin oma pallo
            munPallo.Position = Mouse.PositionOnWorld;
            // Värjätään vanha pallo takaisin valkoiseksi
            lahinPallo.Color = Color.White;
            // Etsitään ja värjätään uusi lähin
            lahinPallo = LahinPallo(pallot, munPallo.Position);
            lahinPallo.Color = Color.Red;
        }, null);

        // TAPA 2: 
        // =======
        // Luodaan erillinen korostuspiste-olio, jota liikutellaan näppäinpainalluksen yhteydessä. 
        // Kutsutaan näppäinkuuntelijassa KorostaLahin-aliohjelmaa ja välitetään sinne tarvittavat muuttujat 
        // (pallot, havaintopiste, korostuspiste) parametreina

        /*
        GameObject korostuspiste = new GameObject(20, 20, Shape.Star);
        korostuspiste.Color = Color.Red;
        Add(korostuspiste);
        PhysicsObject lahinPallo = LahinPallo(pallot, munPallo.Position);
        korostuspiste.Position = lahinPallo.Position;

        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, KorostaLahin, "Muuta havaintopisteen sijaintia", pallot, munPallo, korostuspiste);
        */

        // TAPA 3: 
        // =======
        // Tehdään aliohjelma (Siirrä) Begin-aliohjelman
        // sisälle ja kutsua sitä hiiren painalluksen yhteydessä

        /*
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, Siirra, null);
        void Siirra()
        {
            munPallo.Position = Mouse.PositionOnWorld;
            lahinPallo.Color = Color.White;
            lahinPallo = LahinPallo(pallot, munPallo.Position);
            lahinPallo.Color = Color.Red;
        }
        */
        
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Poistaa korostuksen vanhasta lähimmästä
    /// ja sen jälkeen korostaa lähimmän pisteen.
    /// </summary>
    /// <param name="pallot">Pallot</param>
    /// <param name="munPallo">Klikattu pallo</param>
    /// <param name="korostuspiste">Erityinen korostusolio</param>
    public void KorostaLahin(PhysicsObject[] pallot, GameObject munPallo, GameObject korostuspiste)
    {
        munPallo.Position = Mouse.PositionOnWorld;
        PhysicsObject lahinPiste = LahinPallo(pallot, munPallo.Position);
        korostuspiste.Position = lahinPiste.Position;
    }

    /// <summary>
    /// Etsitään fysiikkaolioiden taulukosta sellainen olio,
    /// joka on lähimpänä annettua parametrivektoria.
    /// Palautetaan lähimmän pallon olio
    /// </summary>
    /// <param name="pallot"></param>
    /// <param name="piste"></param>
    /// <returns></returns>
    public static PhysicsObject LahinPallo(PhysicsObject[] pallot, Vector piste)
    {
        PhysicsObject lahin = pallot[0];
        foreach (PhysicsObject pallo in pallot)
        {
            if (Vector.Distance(pallo.Position, piste) < Vector.Distance(lahin.Position, piste))
            {
                lahin = pallo;
            }
        }
        return lahin;
    }

    /// <summary>
    /// Tekee palloja satunnaisiin paikkoihin
    /// </summary>
    /// <param name="montako">Montako palloa tehdään</param>
    /// <param name="koko">Pallon koko</param>
    /// <returns>Pallot</returns>
    private PhysicsObject[] TeeSatunnaisetPallot(int montako, double koko)
    {
        PhysicsObject[] pallot = new PhysicsObject[montako];
        for (int i = 0; i < montako; i++)
        {
            PhysicsObject p = new PhysicsObject(koko, koko, Shape.Circle);
            pallot[i] = p;
            p.Position = RandomGen.NextVector(Level.BoundingRect);
            Add(p);
        }
        return pallot;
    }
}
