using System;
using System.Collections.Generic;
using System.Linq;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Moodi-keskiluku.
/// </summary>
public class B2_Moodi
{
    /// <summary>
    /// Pääohjelma
    /// </summary>
    public static void Main()
    {
        int[] luvut = { 1, 3, 3, 2, 5, 5, 2, 5 };
        int m1 = Moodi(luvut); // 5
        int m2 = Moodi(new int[] { 1 }); // 1
        int m3 = Moodi(new int[] { 3, 3 }); // 3
        int m4 = Moodi(new int[] { }); // 0
        int m5 = Moodi(new int[] { 2, 2, 2, 3, 3, 3, 4, 1 }); // 2

        int m6 = Moodi3(luvut); // Moodi linq:lla

        Console.WriteLine(m1);
        Console.WriteLine(m2);
        Console.WriteLine(m3);
        Console.WriteLine(m4);
        Console.WriteLine(m5);
        Console.WriteLine(m6);
    }

    /// <summary>
    /// Etsii kokonaislukutaulukon moodin eli useimmin esiintyvän alkion.
    /// Jos eniten esiintyviä on useita, niin ensimmäisenä löytynyt tulkitaan moodiksi.
    /// </summary>
    /// <param name="luvut">Kokonaislukutaulukko, josta etsitään moodi</param>
    /// <returns>Useimmin esiintynyt alkio</returns>
    public static int Moodi(int[] luvut)
    {
        if (luvut.Length == 0) return 0;
        int moodi = luvut[0];
        int esiintymia = 0;

        for (int i = 0; i < luvut.Length; i++)
        {
            int nykyinen = luvut[i];
            int nykyisenEsiintymat = 0;
            for (int j = 0; j < luvut.Length; j++)
            {
                if (luvut[j] == nykyinen) nykyisenEsiintymat++;
            }

            if (nykyisenEsiintymat > esiintymia)
            {
                moodi = nykyinen;
                esiintymia = nykyisenEsiintymat;
            }
        }

        return moodi;
    }

    /// <summary>
    /// EXTRA-TIETOA KIINNOSTUNEILLE:
    /// Moodi sanakirjaa käyttämällä.
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <returns></returns>
    public static int Moodi2(int[] luvut)
    {
        var frekvenssit = new Dictionary<int, int>();
        foreach (var luku in luvut)
        {
            if (frekvenssit.ContainsKey(luku)) frekvenssit[luku]++;
            else frekvenssit.Add(luku, 1);
        }

        // Etsitään se avain (key), jonka arvo (value) on suurin
        // Tehdään alkuarvaus, että sanakirjan ensimmäinen alkio
        // on suurin.
        KeyValuePair<int, int> maksimi = frekvenssit.First();
        foreach (var frekvenssi in frekvenssit)
        {
            if (frekvenssi.Value > maksimi.Value) maksimi = frekvenssi;
        }
        return maksimi.Key;
        
        // Maksimin voi etsiä myös MaxBy-metodilla.
        // return frekvenssit.MaxBy(x => x.Value).Value;
    }

    /// <summary>
    /// Moodi käyttäen LINQ:ta. 
    /// </summary>
    /// <param name="luvut">Luvut.</param>
    /// <returns>Moodi.</returns>
    public static int Moodi3(int[] luvut)
    {
        var alkiotYleisyysjarjestyksessa =
            luvut.GroupBy(x => x).OrderByDescending(x => x.Count()).Select(x => x.Key).ToList();
        return alkiotYleisyysjarjestyksessa[0];
    }
}