using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Sisältääkö jono x jonon y kaikki merkit.
/// </summary>
public class B3_SisaltaakoKaikkiMerkit
{
    public static void Main()
    {
        string[] jonot = { "antti", "antti", "at", "anttijussi", "anttijussi", "anttijussi", "", "" };
        string[] upotettavat = { "antti", "at", "antti", "tia", "", "tiaa", "", "a" };

        for (int i = 0; i < jonot.Length; i++)
        {
            Console.Write("\"" + jonot[i] + "\", \"" + upotettavat[i] + "\" --> ");
            if (SisaltaaKaikkiMerkit(jonot[i], upotettavat[i]))
            {
                Console.WriteLine("ok");
            }
            else
            {
                Console.WriteLine("ei hyva");
            }
        }


    }

    /// <summary>
    /// Funktiossa tutkitaan 
    /// sisaltaako merkkijono kaikki annetut merkit. 
    /// </summary>
    /// <param name="jono">Merkkijono</param>
    /// <param name="merkit">Upotettavat merkit (eli tämän jonon 
    /// merkit tulee löytyä ekasta jonosta.</param>
    /// <returns>Löytyykö merkit</returns>
    /// <pre name="test">
    /// SisaltaaKaikkiMerkit("antti", "antti") === true;
    /// SisaltaaKaikkiMerkit("antti", "anttti") === false;
    /// SisaltaaKaikkiMerkit("antti", "at") === true;
    /// SisaltaaKaikkiMerkit("at", "antti") === false;
    /// SisaltaaKaikkiMerkit("anttijussi", "tia") === true;
    /// SisaltaaKaikkiMerkit("anttijussi", "") === true;
    /// SisaltaaKaikkiMerkit("anttijussi", "tiaa") === false;
    /// SisaltaaKaikkiMerkit("", "") === true;
    /// SisaltaaKaikkiMerkit("", "a") === false;
    /// </pre>
    /// </example>
    public static bool SisaltaaKaikkiMerkit(string jono, string merkit)
    {
        StringBuilder sanaNyt = new StringBuilder(jono);
        int i = 0;
        while (i < merkit.Length)
        {
            char merkkiNyt = merkit[i];
            // Seuraavaksi etsitään tämä kyseinen merkki
            // sanaNyt-merkkijonossa

            // int merkinPaikka = sanaNyt.IndexOf(merkkiNyt); // voitaisiin toteuttaa myös laajennusmetodi. Toteutus alla.

            int merkinPaikka = -1; // arvataan aluksi että merkkiä ei löydy
            int j = 0;
            while (j < sanaNyt.Length)
            {
                if (sanaNyt[j] == merkkiNyt)
                {
                    merkinPaikka = j;
                    break;
                }
                j++;
            }

            if (merkinPaikka >= 0)
            {
                sanaNyt.Remove(merkinPaikka, 1);
            }
            else return false;
            i++;

        }
        return true;
    }
}

/// <summary>
/// Apuluokka, jolla laajennetaan
/// olioiden toiminnallisuutta ns. laajennusmetodeilla (``extension methods'').
/// </summary>
public static class Helper
{
    /// <summary>
    /// String.IndexOfia vastaava toiminto StringBuilderille.
    /// Tätä ei valitettavasti .NET:stä löydy oletuksena.
    /// Palauttaa indeksin, josta merkki löytyi, ja -1, jos
    /// merkkiä ei löydy.
    /// </summary>
    /// <param name="sb">StringBuilder-olio</param>
    /// <param name="c">Etsittävä merkki</param>
    /// <param name="startIndex">Indeksi, josta lähdetään tutkimaan</param>
    /// <returns>Kohta, josta merkki löytyi, tai -1, 
    /// jos merkkiä ei löydy.</returns>
    public static int IndexOf(this StringBuilder sb, char c, int startIndex = 0)
    {
        for (int i = startIndex; i < sb.Length; i++)
            if (sb[i] == c)
                return i;
        return -1;
    }
}
