using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Tehtävä 4. Anagrammi.
/// </summary> 
public class T4_Anagrammi
{
    public delegate bool AnagrammiDelegate(string jono1, string jono2);

    /// <summary>
    /// Pääohjelma
    /// </summary>
    public static void Main()
    {
        bool a1 = Anagrammi1("Antti", "titnA");
        Console.WriteLine("Antti ja titnA {0} anagrammeja", a1 ? "on" : "ei ole");

        // Ajastetaan eri funktioita: 
        double ms;
        int iteraatioita = 100000000;
        // Ajastetaan eri funktioita: 
        Console.WriteLine("Anagrammi1");
        ms = Ajasta(new AnagrammiDelegate(Anagrammi1), iteraatioita);
        Console.WriteLine(ms + " ms");

        Console.WriteLine("Anagrammi2");
        ms = Ajasta(new AnagrammiDelegate(Anagrammi2), iteraatioita);
        Console.WriteLine(ms + " ms");


        Console.WriteLine("Anagrammi3");
        ms = Ajasta(new AnagrammiDelegate(Anagrammi3), iteraatioita);
        Console.WriteLine(ms + " ms");

    }

    /// <summary>
    /// Anagrammi käyttäen merkkitaulukon (char[]) järjestämistä 
    /// Array.Sortilla. 
    /// </summary>
    /// <param name="jono1">Jono1</param>
    /// <param name="jono2">Jono2</param>
    /// <returns>Onko anagrammi</returns>
    /// <example>
    /// <pre name="test">
    /// Anagrammi1("Antti", "ittnA") === true;
    /// Anagrammi1("antti", "ittnA") === true;
    /// Anagrammi1("an tti", "ittnA") === true;
    /// Anagrammi1("an tti", "i ttnA") === true;
    /// Anagrammi1("anttijussi", "I Just stina") === true;
    /// </pre>
    /// </example>
    public static bool Anagrammi1(string jono1, string jono2)
    {
        jono1 = jono1.Replace(" ", "");
        jono2 = jono2.Replace(" ", "");
        if (jono1.Length != jono2.Length) return false;
        jono1 = jono1.ToLower();
        jono2 = jono2.ToLower();

        // "Antti";
        // "titnA";
        // --> 
        // "Aintt", "Aintt"
        
        // "Antti";
        // "titnAzzz";
        // --> 
        // "Aintt", "Ainttzzz"

        // string jono1a = jono1.Concat()
        char[] jono1taulukkona = jono1.ToCharArray();
        char[] jono2taulukkona = jono2.ToCharArray();
        Array.Sort(jono1taulukkona);
        Array.Sort(jono2taulukkona);

        //string s1 = jono1taulukkona.ToString();
        //string s2 = jono2taulukkona.ToString();

        //if (s1 != s2) return false;
        //else return true;

        for (int i = 0; i < jono1taulukkona.Length; i++)
        {
            if (Char.ToLower(jono1taulukkona[i]) != Char.ToLower(jono2taulukkona[i])) return false;
        }
        return true;
    }


    /// <summary>
    /// Anagrammi Dictionaryllä
    /// </summary>
    /// <param name="jono1">Jono 1.</param>
    /// <param name="jono2">Jono 2.</param>
    /// <returns>Ovatko jonot anagrammeja.</returns>
    /// <example>
    /// <pre name="test">
    /// Anagrammi2("Antti", "ittnA") === true;
    /// Anagrammi2("antti", "ittnA") === true;
    /// Anagrammi2("an tti", "ittnA") === true;
    /// Anagrammi2("an tti", "i ttnA") === true;
    /// Anagrammi2("anttijussi", "I Just stina") === true;
    /// </pre>
    /// </example>
    public static bool Anagrammi2(string jono1, string jono2)
    {
        jono1 = jono1.Replace(" ", "");
        jono2 = jono2.Replace(" ", "");

        jono1 = jono1.ToLower();
        jono2 = jono2.ToLower();

        Dictionary<char, int> esiintymat1 = new Dictionary<char, int>();
        Dictionary<char, int> esiintymat2 = new Dictionary<char, int>();

        foreach (char merkki in jono1)
        {
            if (esiintymat1.ContainsKey(merkki)) esiintymat1[merkki] += 1;
            // Merkkiä ei vielä löydy sanakirjasta
            else esiintymat1.Add(merkki, 1);
        }

        foreach (char merkki in jono2)
        {
            if (esiintymat2.ContainsKey(merkki)) esiintymat2[merkki]++;
            else esiintymat2.Add(merkki, 1);
        }

        // esiintymat1.Equals // Vertaa olioviitteitä?

        if (esiintymat1.Count != esiintymat2.Count) return false;

        foreach (KeyValuePair<char, int> merkinEsiintyma in esiintymat1)
        {
            // Alla oleva tarkoittaa, että jono2:sta ei löydy merkkiä
            // joka jono1:ssä esiintyy
            if (!esiintymat2.ContainsKey(merkinEsiintyma.Key)) return false;
            if (merkinEsiintyma.Value != esiintymat2[merkinEsiintyma.Key]) return false;
        }

        return true;

    }

    public static bool Anagrammi3(string sana1, string sana2)
    {
        return Anagrammi3(sana1, sana2, false);
    }

    /// <summary>
    /// Anagrammi. Tehtävä 4 ja guru 3-5.
    /// Algoritmi toimii siten, että lasketaan, kuinka
    /// monta kertaa kukin kirjain esiintyy sanassa,
    /// ja talletetaan määrät taulukkoon. Sitten vertaillaan
    /// näitä kahdesta sanasta saatuja taulukoita ja
    /// jos ne ovat täsmälleen samat, on kyseessä anagrammit.
    /// Välilyöntejä ja kirjainkokoa ei oteta tässä huomioon.
    /// Asymptoottinen suoritusaika mahdollisesti O(n), tosin 
    /// ToLower saattaa olla hitaampi (ehkä). 
    /// </summary>
    /// <param name="sana1">Sana 1</param>
    /// <param name="sana2">Sana 2</param>
    /// <returns>Ovatko kaksi sanaa anagrammit</returns>
    /// <example>
    /// <pre name="test">
    /// Anagrammi3("Antti", "ittnA") === true;
    /// Anagrammi3("antti", "ittnA") === true;
    /// // Anagrammi3("an tti", "ittnA") === true;
    /// // Anagrammi3("an tti", "i ttnA") === true;
    /// // Anagrammi3("anttijussi", "I Just stina") === true;
    /// </pre>
    /// </example>
    public static bool Anagrammi3(string sana1, string sana2, bool valilyonnitPois = false)
    {
        sana1 = sana1.ToLower();// .Replace(" ", ""); // Tämä saattaa hidastaa algoritmia! Yritän selvittää tuleville kerroille ToLower-metodin aikavaativuuden. 
        sana2 = sana2.ToLower();// .Replace(" ", "");
        // if (sana1.Length != sana2.Length) return false;
        int[] frekvenssitSana1 = new int[(int)char.MaxValue];
        int[] frekvenssitSana2 = new int[(int)char.MaxValue];
        for (int i = 0; i < sana1.Length; i++)
        {
            frekvenssitSana1[(int)sana1[i]]++;
        }


        for (int i = 0; i < sana2.Length; i++)
        {
            frekvenssitSana2[(int)sana2[i]]++;
        }


        if (valilyonnitPois) { frekvenssitSana1[(int)' '] = 0; frekvenssitSana2[(int)' '] = 0; }
        for (int i = 0; i < frekvenssitSana1.Length; i++)
            if (frekvenssitSana1[i] != frekvenssitSana2[i]) return false;
        return true;
    }


    /// <summary>
    /// Kauanko aikaa funktion suorittamiseen menee.
    /// </summary>
    /// <param name="a"></param>
    /// <param name="iteraatioita"></param>
    /// <returns></returns>
    public static double Ajasta(AnagrammiDelegate a, int iteraatioita)
    {
        Stopwatch stopWatch;
        int i = 0;
        StringBuilder sana1 = new StringBuilder();
        StringBuilder sana2 = new StringBuilder();
        bool ovatkoAnagrammeja;
        Random r = new Random();
        while (sana1.Length < iteraatioita)
        {
            char c1 = (char)r.Next(71, 91);
            char c2 = (char)r.Next(71, 91);
            sana1.Append(c1 + "a" + (i * 1 + 1));
            sana2.Append(c2 + "a" + (i * 1 + 1));
            i++;
        }
        string s1 = sana1.ToString(); // muutetaan sanat string-olioiksi ennen ajastusta,
        string s2 = sana2.ToString(); // koska ToString() vie aikaa!

        stopWatch = Stopwatch.StartNew();
        ovatkoAnagrammeja = a(s1, s2);
        stopWatch.Stop();
        double ms = stopWatch.Elapsed.TotalMilliseconds;

        return ms;
    }
}
