using System;
using System.Diagnostics;

/// <summary>
/// Demo 9, tehtävä 4.
/// Etsitään onko annettu luku jokin kahden alkion summa.
/// </summary>
public class T4_EtsiSumma
{
    /// <summary>
    /// Luodaan satunnainen taulukko ja otetaan aikaa kauanko menee 
    /// sopivan summan löytymiseen. 
    /// </summary>
    public static void Main()
    {
        // Valmiiksi TIMissä annettu pääohjelma
        int i1 = 6, i2 = 4;
        int[] t = new int[] { 1, 2, 4, 4, 18, 1, 1 };

        foreach (int i in t)
            Console.Write(i + " ");
        Console.WriteLine();

        Console.WriteLine("Taulukosta " + (OnkoSummaa(t, i1) ? "löytyi summa" : "ei löytynyt summaa") + " luvulle " + i1);
        Console.WriteLine("Taulukosta " + (OnkoSummaa(t, i2) ? "löytyi summa" : "ei löytynyt summaa") + " luvulle " + i2);

        // Performanssitesti
        const int MONTAKO = 100000;
        int[] luvut = LuoSatunnainenTaulukko(MONTAKO, 0, MONTAKO / 2);

        Array.Sort(luvut);

        Stopwatch sw;
        bool loytyikoSumma;
        double ms;

        // Hidas ratkaisu
        sw = Stopwatch.StartNew();
        loytyikoSumma = OnkoSummaa(luvut, MONTAKO);
        sw.Stop();
        ms = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine($"OnkoSummaa, kesti {ms} millisekuntia kun alkioita oli {MONTAKO}.");

        // Nopea ratkaisu
        sw = Stopwatch.StartNew();
        loytyikoSumma = OnkoSummaa2(luvut, MONTAKO);
        sw.Stop();
        ms = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine($"OnkoSummaa2, kesti {ms} millisekuntia kun alkioita oli {MONTAKO}.");
    }

    /// <summary>
    /// Luo satunnaisen taulukon
    /// </summary>
    /// <param name="pituus"></param>
    /// <param name="min"></param>
    /// <param name="max"></param>
    /// <returns></returns>
    private static int[] LuoSatunnainenTaulukko(int pituus, int min, int max)
    {
        Random r = new Random();
        int[] luvut = new int[pituus];
        for (int i = 0; i < luvut.Length; i++)
        {
            luvut[i] = r.Next(min, max);
        }
        return luvut;
    }

    /// <summary>
    /// Löytyykö taulukosta lukuparia, jonka summa on 
    /// annettu summa. Oletetaan että luvut ovat
    /// nousevassa järjestyksessä. 
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <param name="summa">Haluttu summa</param>
    /// <returns>Onko lukuparia joka vastaa summaa</returns>
    /// <example>
    /// <pre name="test">
    /// OnkoSummaa2(new int[] {1, 2, 3, 9}, 8) === false;
    /// OnkoSummaa2(new int[] {1, 2, 4, 4}, 8) === true;
    /// OnkoSummaa2(new int[] {1, 2, 4}, 8) === false;
    /// OnkoSummaa2(new int[] {1, 1}, 1) === false;
    /// OnkoSummaa2(new int[] {1, 1}, 2) === true;
    /// OnkoSummaa2(new int[] {1}, 1) === false;
    /// OnkoSummaa2(new int[] {}, 0) === false;
    /// </pre>
    /// </example>
    public static bool OnkoSummaa2(int[] luvut, int summa)
    {
        Array.Sort(luvut);

        if (luvut.Length < 2) return false;
        int vasen = 0;
        int oikea = luvut.Length - 1;
        while (vasen < oikea)
        {
            int summaNyt = luvut[vasen] + luvut[oikea];
            if (summaNyt == summa) return true;
            if (summaNyt > summa) oikea--;
            if (summaNyt < summa) vasen++;
        }
        return false;
    }

    /// <summary>
    /// Löytyykö taulukosta lukuparia, jonka summa on 
    /// annettu summa. Tämä on aikavaativuudeltaan O(n^2) 
    /// joten ratkaisu on hitaampi isoilla taulukoilla.
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <param name="summa">Haluttu summa</param>
    /// <returns>Onko lukuparia joka vastaa summaa</returns>
    /// <example>
    /// <pre name="test">
    /// OnkoSummaa(new int[] {1, 2, 3, 9}, 8) === false;
    /// OnkoSummaa(new int[] {1, 2, 4, 4}, 8) === true;
    /// OnkoSummaa(new int[] {1, 2, 4}, 8) === false;
    /// OnkoSummaa(new int[] {1, 1}, 1) === false;
    /// OnkoSummaa(new int[] {1, 1}, 2) === true;
    /// OnkoSummaa(new int[] {1}, 1) === false;
    /// OnkoSummaa(new int[] {}, 0) === false;
    /// </pre>
    /// </example>
    public static bool OnkoSummaa(int[] luvut, int summa)
    {
        for (int i = 0; i < luvut.Length; i++)
        {
            for (int j = 0; j < luvut.Length; j++)
            {
                if (i == j) continue;
                if (luvut[i] + luvut[j] == summa) return true;
            }
        }
        return false;
    }

}
