using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.Widgets;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Tehtävä Bonus 1-2. Palloja rekursiivisesti.
/// </summary>
public class T12_Pallopuu : Game
{
    const double PIENIN_SADE = 4.0;

    /// <summary>
    /// Kutsutaan rekursiivista aliohejlmaa.
    /// </summary>
    public override void Begin()
    {
        bool lisahienoudet = true;
        if (!lisahienoudet)
        {
            Pallo(0, 150, 100, Math.PI / 2);
        }
        else
        {

            List<GameObject> pallot = new List<GameObject>();
            Pallo(0, 0, 100, 0.5, Math.PI / 2, Math.PI / 8, pallot);

            DoubleMeter kulmaMeter = new DoubleMeter(Math.PI / 8, 0, Math.PI);
            DoubleMeter suuntaMeter = new DoubleMeter(Math.PI / 2, 0, Math.PI);
            DoubleMeter kokoMeter = new DoubleMeter(0.5, 0.05, 0.7);

            kulmaMeter.Changed += PiirraUudestaan;
            suuntaMeter.Changed += PiirraUudestaan;
            kokoMeter.Changed += PiirraUudestaan;

            void PiirraUudestaan(double vanhaArvo, double uusiArvo)
            {
                pallot.ForEach(x => x.Destroy());
                pallot.Clear();
                Pallo(0, 0, 100, kokoMeter.Value, suuntaMeter.Value, kulmaMeter.Value, pallot);
            };

            Slider kulmaSlider = new Slider(150, 15)
            {
                Position = new Vector(0, -100),
                Color = Color.Black
            };
            kulmaSlider.BindTo(kulmaMeter);
            Add(kulmaSlider);
            Label kulmaLabel = new Label
            {
                DoubleFormatString = "Kulma: +/- {0:N2}",
                Color = Color.Black,
                TextColor = Color.White,
                Position = new Vector(kulmaSlider.X, kulmaSlider.Top + 15)
            };
            kulmaLabel.BindTo(kulmaSlider.Meter);
            Add(kulmaLabel);

            Slider suuntaSlider = new Slider(150, 15)
            {
                Left = kulmaSlider.Left,
                Color = Color.Black
            };
            suuntaSlider.Bottom = kulmaLabel.Top + suuntaSlider.Height * 2;
            Add(suuntaSlider);
            suuntaSlider.BindTo(suuntaMeter);
            Label suuntaLabel = new Label
            {
                TextColor = Color.White,
                Color = Color.Black,
                Position = new Vector(suuntaSlider.X, suuntaSlider.Top + 15),
                DoubleFormatString = "Suunta:  {0:N2}"
            };
            suuntaLabel.BindTo(suuntaSlider.Meter);
            Add(suuntaLabel);

            Slider kokoSlider = new Slider(150, 15)
            {
                Left = suuntaSlider.Left,
                Top = kulmaSlider.Bottom - 60,
                Color = Color.Black
            };
            kokoSlider.BindTo(kokoMeter);
            Add(kokoSlider);
            Label kokoLabel = new Label
            {
                Color = Color.Black,
                TextColor = Color.White,
                Position = new Vector(kokoSlider.X, kokoSlider.Top + 15),
                DoubleFormatString = "Kokokerroin:  {0:N2}"
            };
            kokoLabel.BindTo(kokoSlider.Meter);
            Add(kokoLabel);
        }

        Mouse.IsCursorVisible = true;

        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, null);

        SetWindowSize(800, 600);
        Level.BackgroundColor = Color.DarkBlue;
        Camera.ZoomToAllObjects(100);
    }

    /// <summary>
    /// Tehdaan pallo, ja sen ylapuolelle kaksi muuta palloa
    /// rekursiivisesti.
    /// </summary>
    /// <param name="x">Pallon kp x-koordinaatti</param>
    /// <param name="y">Pallon kp y-koordinaatti</param>
    /// <param name="sade">Pallon sade</param>
    /// <param name="suunta">Suunta radiaaneina</param>
    public void Pallo(double x, double y, double sade, double suunta)
    {
        //jos pallon koko < 4 lopeta
        if (sade < PIENIN_SADE) return;

        //pallo pisteeseen (x,y)
        GameObject pallo = new GameObject(sade * 2, sade * 2, Shape.Circle);
        pallo.Position = new Vector(x, y);
        pallo.Color = RandomGen.NextColor();
        Add(pallo);


        // määrittele uusikoko itse ...
        double r = sade * 0.5;
        // määrittele suunnat itse ...
        double vasenSuunta = suunta + Math.PI / 8;
        double oikeaSuunta = suunta - Math.PI / 8;

        Pallo(x + Math.Cos(vasenSuunta) * sade + Math.Cos(vasenSuunta)
            * r, y + Math.Sin(vasenSuunta) * sade + Math.Sin(vasenSuunta)
            * r, r, vasenSuunta);

        Pallo(x + Math.Cos(oikeaSuunta) * sade + Math.Cos(oikeaSuunta)
            * r, y + Math.Sin(oikeaSuunta) * sade + Math.Sin(oikeaSuunta)
            * r, r, oikeaSuunta);
    }

    /// <summary>
    /// Tehdaan pallo, ja sen ylapuolelle kaksi muuta palloa
    /// rekursiivisesti.
    /// </summary>
    /// <param name="x">Pallon kp x-koordinaatti</param>
    /// <param name="y">Pallon kp y-koordinaatti</param>
    /// <param name="sade">Pallon sade</param>
    /// <param name="sadeKerroin">Sädekerroin</param>
    /// <param name="suunta">Suunta radiaaneina</param>
    /// <param name="kulma">Kulma</param>
    /// <param name="pallot">Pallot-lista</param>
    public void Pallo(double x, double y, double sade, double sadeKerroin, double suunta, double kulma, List<GameObject> pallot)
    {
        //pallo pisteeseen (x,y)
        GameObject pallo = new GameObject(sade * 2, sade * 2, Shape.Circle);
        pallo.Position = new Vector(x, y);
        Add(pallo);
        pallot.Add(pallo);

        if (sade < PIENIN_SADE) return;
        if (pallot.Count > 500) return;
        //jos pallon koko < 4 lopeta

        // määrittele uusikoko itse ...
        double r = sade * sadeKerroin;
        // määrittele suunnat itse ...
        double vasenSuunta = suunta + kulma;
        double oikeaSuunta = suunta - kulma;

        Pallo(x + Math.Cos(vasenSuunta) * sade + Math.Cos(vasenSuunta)
            * r, y + Math.Sin(vasenSuunta) * sade + Math.Sin(vasenSuunta)
            * r, r, sadeKerroin, vasenSuunta, kulma, pallot);

        Pallo(x + Math.Cos(oikeaSuunta) * sade + Math.Cos(oikeaSuunta)
            * r, y + Math.Sin(oikeaSuunta) * sade + Math.Sin(oikeaSuunta)
            * r, r, sadeKerroin, oikeaSuunta, kulma, pallot);
    }

}