using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Numerics;

/// <summary>
/// Tehtävä 4-5. Lasketaan n yli k eli k-kombinaatioita.
/// </summary>
public class T45_NYliK
{
    /// <summary>
    /// Aliohjelmakutsuja ja tulostusta.
    /// </summary>
    /// <param name="args">Ei kaytossa.</param>
    public static void Main(string[] args)
    {
        long l = NYliK(40, 7);
        Console.WriteLine(l);
        BigInteger b = BigNYliK(40, 7);
        Console.WriteLine(b);


        // Esimerkki lukualueen ylityksestä. 

        int a = int.MaxValue;
        Console.WriteLine(a);
        a++; // lukualueen ylitys. Huom! Ei käännösvirhe eikä ajonaikainen virhe.
        Console.WriteLine(a);

        // 40! käyttäen BigIntegeriä.
        // BigInteger bi = new BigInteger(1);
        // for (int i = 1; i <= 40; i++)
        // {
        //     bi *= i;
        // }
        // Console.WriteLine(bi.ToString());
    }

    /// <summary>
    /// Antaa n yli k:n, ts. C(n, k).
    /// Ks. http://fi.wikipedia.org/wiki/Kombinaatio
    /// Esim. kuinka monta erilaista 7 luvun ryhmää voidaan
    /// muodostaa 40 luvun joukosta:
    ///  40         40!          34*35*36*37*38*39*40
    /// (  )  = ------------  =  -------------------- = 18643560
    ///   7     7! * (40-7)!        1*2*3*4*5*6*7
    ///   
    /// Aliohjelma laskee tuloksen käyttäen viimeistä
    /// muotoa.
    /// </summary>
    /// <param name="n">n</param>
    /// <param name="k">k</param>
    /// <returns>n yli k:n</returns>
    /// <example>
    /// <pre name="test">
    /// Kombinaatio.NYliK(40, 7) === 18643560;
    /// Kombinaatio.NYliK(39, 7) === 15380937;
    /// Kombinaatio.NYliK(1, 1) === 1;
    /// Kombinaatio.NYliK(1, 0) === 1;
    /// Kombinaatio.NYliK(1, 0) === 1;
    /// </pre>
    /// </example>
    public static long NYliK(int n, int k)
    {
        long osoittaja = Kerro(n - k + 1, n);
        long nimittaja = Kerro(1, k);
        return osoittaja / nimittaja;
    }

    /// <summary>
    /// Kerrotaan keskenään luvut alku, alku+1, ..., loppu.
    /// Jos loppu > alku, palautetaan 1. Lukujen ollessa
    /// yhtä suuria palautetaan alku.
    /// </summary>
    /// <param name="alku">Mistä luvusta aloitetaan.</param>
    /// <param name="loppu">Mihin lukuun lopetetaan.</param>
    /// <returns>Lukujen tulo.</returns>
    /// <example>
    /// <pre name="test">
    /// Kerro(1, 1) === 1;
    /// Kerro(1, 5) === 120;
    /// Kerro(7, 10) === 5040;
    /// Kerro(0, 5) === 0;
    /// Kerro(3, 1) === 1;
    /// Kerro(5, 2) === 1;
    /// Kerro(5, 5) === 5;
    /// </pre>
    /// </example>
    public static long Kerro(int alku, int loppu)
    {
        long tulos = 1;
        for (int i = alku; i <= loppu; i++)
        {
            tulos *= i; // Esimerkiksi osoittaja meidän esimerkissämme: 34 * 35 * ... * 40
            // (Tai nimittäjä meidän esimerkissämme: 1 * 2 * ... * 7)
        }
        return tulos;
    }

    /// <summary>
    /// Kerrotaan keskenään luvut alku, alku+1, ..., loppu.
    /// Jos loppu > alku, palautetaan 1. Lukujen ollessa
    /// yhtä suuria palautetaan alku.
    /// http://msdn.microsoft.com/en-us/library/system.numerics.biginteger.aspx
    /// </summary>
    /// <param name="alku">Mistä luvusta aloitetaan.</param>
    /// <param name="loppu">Mihin lukuun lopetetaan.</param>
    /// <returns>Lukujen tulo.</returns>
    /// <example>     
    /// <pre name="test">
    /// Kerro(1, 1) === (BigInteger) 1;
    /// Kerro(1, 5) === (BigInteger) 120;
    /// Kerro(7, 10) === (BigInteger) 5040;
    /// Kerro(0, 5) === (BigInteger) 0;
    /// Kerro(1, 0) === (BigInteger) 1;
    /// Kerro(3, 1) === (BigInteger) 1;
    /// Kerro(5, 2) === (BigInteger) 1;
    /// Kerro(5, 5) === (BigInteger) 5;
    /// </pre>
    /// </example>     
    public static BigInteger BigKerro(int alku, int loppu)
    {
        BigInteger tulos = 1;
        for (int i = alku; i <= loppu; i++)
        {
            tulos *= i;
        }
        return tulos;
    }

    /// <summary>
    /// Antaa n yli k:n, ts. C(n, k).
    /// Ks. http://fi.wikipedia.org/wiki/Kombinaatio
    /// Esim. kuinka monta erilaista 7 luvun ryhmää voidaan
    /// muodostaa 40 luvun joukosta:
    ///  40         40!          34*35*36*37*38*39*40
    /// (  )  = ------------  =  -------------------- = 18643560
    ///   7     7! * (40-7)!        1*2*3*4*5*6*7
    ///   
    /// Aliohjelma laskee tuloksen käyttäen viimeistä
    /// muotoa.
    /// </summary>
    /// <param name="n">n</param>
    /// <param name="k">k</param>
    /// <returns>n yli k:n</returns>
    /// <example>     
    /// <pre name="test">
    /// NYliK(40, 7) === (BigInteger) 18643560;
    /// NYliK(39, 7) === (BigInteger) 15380937;
    /// NYliK(1, 1) === (BigInteger) 1;
    /// NYliK(1, 0) === (BigInteger) 1;
    /// NYliK(0, 1) === (BigInteger) 0;
    /// </pre>
    /// </example>     
    public static BigInteger BigNYliK(int n, int k)
    {
        BigInteger osoittaja = BigKerro(n - k + 1, n);
        BigInteger nimittaja = BigKerro(1, k);
        return osoittaja / nimittaja;
    }
}
