using Jypeli;
using System;
using System.Collections.Generic;
using System.Text;

namespace Jypeli
{
    public static class ImageExtension
    {
        /// <summary>
        /// Asettaa Image-kuvan pikselit
        /// byte-taulukosta.
        /// </summary>
        /// <param name="image"></param>
        /// <param name="kuvaTavut"></param>
        /// <param name="kuvanKorkeus"></param>
        /// <param name="kuvanLeveys"></param>
        public static void SetData(this Image image, byte[] kuvaTavut, int kuvanKorkeus, int kuvanLeveys)
        {
            Color[,] varidata = new Color[kuvanKorkeus, kuvanLeveys];

            for (int i = 0; i < kuvanKorkeus; i++)
            {
                for (int j = 0; j < kuvanLeveys; j++)
                {
                    int r = kuvaTavut[4 * (i * kuvanLeveys + j) + 0];
                    int g = kuvaTavut[4 * (i * kuvanLeveys + j) + 1];
                    int b = kuvaTavut[4 * (i * kuvanLeveys + j) + 2];
                    int a = kuvaTavut[4 * (i * kuvanLeveys + j) + 3];
                    varidata[i, j] = new Color(r, g, b, a);
                }
            }
            image.SetData(varidata);
        }

    }
}

/// <summary>
/// Salakirjoitetaan viesti kuvaan.
/// Luetaan kuvasta salakirjoitettu viesti. 
/// </summary>
public class G10_Salakirjoitus : PhysicsGame
{
    public override void Begin()
    {
        Level.BackgroundColor = Color.Black;

        Image image1 = LoadImage("soodaaja");
        Image image2 = LoadImage("soodaaja");
        Add(new GameObject(image1) { Position = new Vector(-100, 100) });
        Add(new GameObject(image2) { Position = new Vector(100, 100) });

        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

        char loppu = (char)0xff;
        byte[] salakirjoitettu = Salakirjoita(image1.GetByteArray(), "Tämä on minun salaviestini" + loppu);

        int kuvanKorkeus = image1.Height;
        int kuvanLeveys = image1.Width;
        image2.SetData(salakirjoitettu, kuvanKorkeus, kuvanLeveys);

        byte[] image1Bytes = image1.GetByteArray();
        byte[] image2Bytes = image2.GetByteArray();

        Image image3 = new Image(kuvanLeveys, kuvanKorkeus, new Color(127, 127, 127, 127));
        byte[] image3Bytes = image3.GetByteArray();
        LisaaErotus(image3Bytes, image1Bytes, image2Bytes);

        image3.SetData(image3Bytes, kuvanKorkeus, kuvanLeveys);
        image3Bytes = image3.GetByteArray();
        Add(new GameObject(image3) { Position = new Vector(0, -200) });

        string viesti = PuraSalaus(salakirjoitettu);
        MessageDisplay.Add(viesti);
    }


    private void LisaaErotus(byte[] mihinLisataan, byte[] image1Bytes, byte[] image2Bytes)
    {
        byte[] erotus = new byte[image1Bytes.Length];
        for (int i = 0; i < image1Bytes.Length; i++)
        {
            mihinLisataan[i] += (byte)((int)image2Bytes[i] - (int)image1Bytes[i]);
        }
    }

    /// <summary>
    /// Salakirjoita viesti byte-taulukkona olevaan kuvaan.
    /// </summary>
    /// <param name="kuva">pikselit neljän byten jonoissa (r, g, b, a).</param>
    /// <param name="viesti">Salakirjoitettava viesti.</param>
    /// <returns></returns>
    public static byte[] Salakirjoita(byte[] kuva, string viesti)
    {
        byte[] salakirjoitettu = new byte[kuva.Length];
        Array.Copy(kuva, salakirjoitettu, kuva.Length);

        for (int i = 0; i < viesti.Length && i * 4 < kuva.Length; i++)
        {
            int u = viesti[i];
            int lower = u & 0xff;

            // Maski:
            // 0x3  00000011

            int bits01 = lower & 0x3; // oikealta lähtien
            lower = lower >> 2;

            int bits23 = lower & 0x3;
            lower = lower >> 2;

            int bits45 = lower & 0x3;
            lower = lower >> 2;

            int bits67 = lower & 0x3;

            // Maskit:
            // 0xfc 11111100

            salakirjoitettu[i * 4] = (byte)(((int)(kuva[i * 4]) & 0xfc) | bits01);
            salakirjoitettu[i * 4 + 1] = (byte)(((int)(kuva[i * 4 + 1]) & 0xfc) | bits23);
            salakirjoitettu[i * 4 + 2] = (byte)(((int)(kuva[i * 4 + 2]) & 0xfc) | bits45);
            salakirjoitettu[i * 4 + 3] = (byte)(((int)(kuva[i * 4 + 3]) & 0xfc) | bits67);
        }
        return salakirjoitettu;
    }

    /// <summary>
    /// Lukee kaksi vähiten merkitsevää bittiä.
    /// (c) Mikko Röyskö
    /// </summary>
    /// <param name="b1"></param>
    /// <param name="b2"></param>
    public static void Lue2Bittia(byte b1, ref byte b2)
    {
        b2 = (byte)(b2 >> 2); // Siirretään jo valmiiksi olevia bittejä 2 askelta oikealle, eli 1100 0000 -> 0011 0000, nyt voidaan seuraavat kaksi bittiä sijoittaa alkuun.
        b1 = (byte)(b1 << 6); // Koska halutaan lukea vain kaksi vähiten merkitsevää, siirretään kaikkea kuusi askelta vasemmalle.
        b2 |= b1;             // Orrilla sijoitetaan nämä kaksi bittiä.
    }

    /// <summary>
    /// Purkaa salauksen kuvan tavutaulukosta.
    /// (c) Mikko Röyskö
    /// </summary>
    /// <param name="kuva"></param>
    /// <returns></returns>
    public static string PuraSalaus(byte[] kuva)
    {
        byte merkki = 0;
        List<byte> mjono = new List<byte>();
        for (int i = 0; i < kuva.Length; i += 4)
        {
            Lue2Bittia(kuva[i], ref merkki);
            Lue2Bittia(kuva[i + 1], ref merkki);
            Lue2Bittia(kuva[i + 2], ref merkki);
            Lue2Bittia(kuva[i + 3], ref merkki);
            if ((byte)merkki == 0xff) break;
            mjono.Add(merkki);
            merkki = 0;
        }
        string lause = Encoding.UTF7.GetString(mjono.ToArray());
        return lause;
    }

}
